
%let np=norow nocol nopercent nocum;

* table shows one cell woth a zero, but no marginal zeroes;
proc freq data=sashelp.class;
table sex*age / &np;
run;

*****************************************************;

* create a format that has categories not present in the data set SASHELP.CLASS;
proc format;
value $sex 'M'='Male' 'F'='Female' 'U'='Unknown' ;
value age low-10 = '<11' 11-16='OK' 17-high='17+';
run;

* still no marginal zeroes;
title 'NO MARGINAL ZEROES';
proc freq data=sashelp.class;
table sex*age / &np;
format sex $sex. age age.;
run;

* use PROC SUMMARY to do the counting;
proc summary data=sashelp.class;
class sex age;
output out=x;
format sex $sex. age age.;
run;

title 'NO ZERO COUNTS';
proc print data=x;
run;

* use a PRELOADED FORMAT to make output conform to a format, not to the data;
proc summary data=sashelp.class completetypes nway;
class sex age / preloadfmt;
output out=x (drop=_type_);
format sex $sex. age age.;
run;

title 'PROC SUMMARY WITH ZERO COUNTS';
proc print data=x;
run;

title 'PROC FREQ WITH ZERO MARGINALS';
proc freq data=x;
table sex*age / &np;
weight _freq_ / zero;
run;

* don't drop _TYPE_ and do not use the NWAY option;
proc summary data=sashelp.class completetypes;
class sex age / preloadfmt;
output out=x;
format sex $sex. age age.;
run;

title 'PROC SUMMARY OUTPUT ... LOOK AT VALUES OF _TYPE_';
proc print data=x;
run;

title 'USE A WHERE STATEMENT TO SELECT OBSERVATIONS FROM PROC SUMMARY';
proc freq data=x;
table sex*age / &np;
weight _freq_ / zero;
where _type_ eq 3;
run;

*****************************************************;

title 'HEIGHT & WEIGHT IN SASHELP.CLASS';
proc freq data=sashelp.class;
table height weight;
run;

* add FORMATS for HEIGHT and WEIGHT ... some categories not in SASHELP.CLASS;
proc format;
value $sex 'M'='Male' 'F'='Female' 'U'='Unknown' ;
value age low-10='<11' 11-16='OK' 17-high='17+';
value weight low-<50='<50' 50-<80='50-<80' 80-<100='80-<100' 100-<150='100-<150' 150-high='150+';
value height low-<50='<50' 50-<80='50-<80' 80-high='80+';
run;

* no NWAY option with THREE variables;
proc summary data=sashelp.class completetypes print;
class height sex age  / preloadfmt;
output out=x;
format sex $sex. age age. height height.;
run;

* take a look at the output from PROC SUMMARY;
* there are multiple set of counts;
title 'PROC SUMMARY OUTPUT WITH THREE VARIABLES';
proc print data=x;
run;

* formats used to reduce output size, make it easier to select TYPES;
proc format;
value $ch ' '=' ' other='*';
value nm  low-high='*' other=' ';
run;

title "PROC SUMMARY DATA SET WITH 3 VARIABLES (2**3 = 8 CATEGORIES)";
proc freq data=x;
tables _type_*height*sex*age / list missing nocum nopercent;
format _character_ $ch. _numeric_ nm. _type_ 2.;
run;
title;

* you can use parts of the PROC SUMMARY output by using a WHERE statement;
title 'USE A WHERE STATEMENT TO SELECT OBSERVATIONS FROM PROC SUMMARY';
proc freq data=x;
table sex*age / &np;
weight _freq_ / zero;
where _type_ eq 3;
run;

* THREE variables;
title 'SPECIFY A TABLE WITH THREE VARIABLES, USE TYPE IN A WHERE STATEMENT';
proc freq data=x;
table height*sex*age / &np;
weight _freq_ / zero;
where _type_ eq 7;
run;

* use CROSSLIST;
title 'SAME TABLE USING THE CROSSLIST OPTION';
proc freq data=x;
table height*sex*age / &np crosslist;
weight _freq_ / zero;
where _type_ eq 7;
run;

*****************************************************;

* no NWAY option with FOUR variables;
proc summary data=sashelp.class completetypes print;
class weight height sex age  / preloadfmt;
output out=x;
format sex $sex. age age. height height. weight weight.;
run;

* take a look at the output from PROC SUMMARY;
* there are multiple set of counts;
title 'PROC SUMMARY OUTPUT WITH FOUR VARIABLES';
proc print data=x;
run;

title "PROC SUMMARY DATA SET WITH 4 VARIABLES (2**4 = 16 CATEGORIES)";
proc freq data=x;
tables _type_*weight*height*sex*age / list missing nocum nopercent;
format _character_ $ch. _numeric_ nm. _type_ 2.;
run;

* you can use parts of the PROC SUMMARY output by using a WHERE statement;
title 'TABLE OF SEX*AGE IS STILL TYPE = 3";
proc freq data=x;
table sex*age / &np;
weight _freq_ / zero;
where _type_ eq 3;
run;

title "TABLE OF HEIGHT*WEIGHT AND TYPE = 12"
proc freq data=x;
table weight*height / &np;
weight _freq_ / zero;
where _type_ eq 12;
run;


*****************************************************;

* ZERO MARGINAL TOTALS, SPARSE OPTION

title "ALREADY SHOWED HOW TO GET ZERO MARGINALS IN PROC FREQ";
proc freq data=x;
table height*sex*age / list &np;
weight _freq_ / zero;
where _type_ eq 7;
run;

title "ONE CELL WITH A ZERO";
proc freq data=sashelp.class;
table sex*age / &np;
run;

title "CHANGE OUTPUT LAYOUT TO LIST ... NO ZERO CELL";
proc freq data=sashelp.class;
table sex*age / list &np;
run;

title "CHANGE OUTPUT LAYOUT TO LIST ... SPARSE OPTION ADDS ZERO CELL";
proc freq data=sashelp.class;
table sex*age / list &np sparse;
run;

title "USE CROSSLIST ... ZERO CELL, NO SPARSE OPTION NEEDED";
proc freq data=sashelp.class;
table sex*age / crosslist;
run;

*****************************************************;

* MISSING and MISSPRINT;

* add some missing data to SASHELP.CLASS;
data class;
set sashelp.class;
if ranuni(0) lt .2 then call missing(age, sex);
/*if ranuni(0) lt .2 then age = .;*/
/*else*/
/*if ranuni(0) lt .3 then sex = ' ';*/
run;

title 'NEW DATA SET WORK.CLASS WITH SOME MIASSING DATA';
proc print data=class;
run;

title 'PROC FREQ WITH THREE DIFFERENT LOOKS";
proc freq data=class;
table sex*age / &np;
table sex*age / missing;
table sex*age / missprint;
run;

title;